<?php

declare(strict_types=1);

namespace SB\Test\Routing\Aspect;

use TYPO3\CMS\Core\Routing\Aspect\StaticMappableAspectInterface;
use TYPO3\CMS\Core\Site\SiteLanguageAwareTrait;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Database\Connection;
use TYPO3\CMS\Core\Utility\GeneralUtility;

class CategoriesValueMapper implements StaticMappableAspectInterface
{
    use SiteLanguageAwareTrait;

    /**
     * {@inheritdoc}
     */
    public function generate(string $value): ?string
    {
        //DebuggerUtility::var_dump($value, 'generate');
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('sys_category');
        $categories = $queryBuilder->select('uid', 'slug')
            ->from('sys_category')
            ->where(
                $queryBuilder->expr()->in('uid', $queryBuilder->createNamedParameter(GeneralUtility::intExplode(',', $value, true), Connection::PARAM_INT_ARRAY)),
            )
            ->orderBy('parent')
            ->execute()
            ->fetchAll();
        //DebuggerUtility::var_dump($categories);

        $routing = $categories[0]['slug'];
        if($categories[1]) {
            $routing .= "_" . $categories[1]['slug'];
        }

        if (isset($routing)) {
            return (string)$routing;
        }
        return null;
    }

    /**
     * {@inheritdoc}
     */
    public function resolve(string $value): ?string
    {
        //DebuggerUtility::var_dump($value, 'resolve');
        $categorieIDsArray = explode('_',$value);
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('sys_category');
        $categories = $queryBuilder->select('uid', 'slug')
            ->from('sys_category')
            ->where(
                $queryBuilder->expr()->in('slug', $queryBuilder->createNamedParameter($categorieIDsArray, Connection::PARAM_STR_ARRAY)),
            )
            ->orderBy('parent')
            ->execute()
            ->fetchAll();
        $routing = strval($categories[0]['uid']);
        if($categories[1]) {
            $routing .= "," . strval($categories[1]['uid']);
        }

        if (isset($routing)) {
            return (string)$routing;
        }
        return null;
    }
}